"""
Example of subscribing to a D-Bus signal using blocking I/O.
This subscribes to the signal for a desktop notification being closed.

To try it, start this script, then trigger a desktop notification, and close it
somehow to trigger the signal. Use Ctrl-C to stop the script.

This example relies on the ``org.freedesktop.Notifications.NotificationClosed``
signal; some desktops may not support it. See the notification spec for more
details:
https://people.gnome.org/~mccann/docs/notification-spec/notification-spec-latest.html

Match rules are defined in the D-Bus specification:
https://dbus.freedesktop.org/doc/dbus-specification.html#message-bus-routing-match-rules
"""

from jeepney.bus_messages import MatchRule, message_bus
from jeepney.integrate.blocking import connect_and_authenticate, Proxy
from jeepney.wrappers import DBusAddress

noti = DBusAddress('/org/freedesktop/Notifications',
                   bus_name='org.freedesktop.Notifications',
                   interface='org.freedesktop.Notifications')

connection = connect_and_authenticate(bus="SESSION")

match_rule = MatchRule(
    type="signal",
    sender=noti.bus_name,
    interface=noti.interface,
    member="NotificationClosed",
    path=noti.object_path,
)

# This defines messages for talking to the D-Bus bus daemon itself:
session_bus = Proxy(message_bus, connection)

# Tell the session bus to pass us matching signal messages:
print("Match added?", session_bus.AddMatch(match_rule) == ())

reasons = {1: 'expiry', 2: 'dismissal', 3: 'dbus', '4': 'undefined'}
def notification_closed(data):
    """Callback for when we receive a notification closed signal"""
    nid, reason_no = data
    reason = reasons.get(reason_no, 'unknown')
    print('Notification {} closed by: {}'.format(nid, reason))

# Connect the callback to the relevant signal
connection.router.subscribe_signal(
    callback=notification_closed,
    path=noti.object_path,
    interface=noti.interface,
    member="NotificationClosed"
)

# Using dbus-send or d-feet or blocking_notify.py, send a notification and
# manually close it or call ``.CloseNotification`` after a beat.
try:
    while True:
        connection.recv_messages()
except KeyboardInterrupt:
    pass



import asyncio

from jeepney.integrate.asyncio import connect_and_authenticate

async def hello():
    (t, p) = await connect_and_authenticate(bus='SESSION')
    print('My ID is:', p.unique_name)
    
loop = asyncio.get_event_loop()
loop.run_until_complete(hello())



"""
This example simulates two clients interacting with the message bus, more or
less independently.

Client 1
    Any app on the bus (here, called *service*). It asks the bus for sole
    custody of its preferred name, a `well-known bus name`_ that it wants
    others to recognize.

Client 2
    An interested party (here, called *watcher*). It asks the bus to emit a
    signal (send it an update) whenever *service*'s well-known bus name changes
    hands.  It uses Jeepney's ``router.subscribe_signal`` to fire a callback
    every time this occurs.

.. _well-known bus name: https://dbus.freedesktop.org/doc
   /dbus-specification.html#message-protocol-names
"""

import asyncio

from jeepney.integrate.asyncio import connect_and_authenticate, Proxy
from jeepney.bus_messages import message_bus, MatchRule

well_known_bus_name = "io.readthedocs.jeepney.aio_subscribe_example"


def print_match(match_info):
    """A demo callback triggered on successful matches."""
    print("[watcher] match hit:", match_info)


async def main():
    __, service_proto = await connect_and_authenticate("SESSION")
    __, watcher_proto = await connect_and_authenticate("SESSION")
    # Interface objects used by each app to interact with the message bus
    service = Proxy(message_bus, service_proto)
    watcher = Proxy(message_bus, watcher_proto)

    # Create a "signal-selection" match rule
    match_rule = MatchRule(
        type="signal",
        sender=message_bus.bus_name,
        interface=message_bus.interface,
        member="NameOwnerChanged",
        path=message_bus.object_path,
    )

    # Condition: arg number 0 must match the bus name (try changing either)
    match_rule.add_arg_condition(0, well_known_bus_name)

    # Register a callback
    watcher_proto.router.subscribe_signal(
        callback=print_match,
        path=message_bus.object_path,
        interface=message_bus.interface,
        member="NameOwnerChanged"
    )

    # Tell the session bus to pass us matching signal messages.
    print("[watcher] adding match rule")
    await watcher.AddMatch(match_rule)
    await asyncio.sleep(1)

    print("[service] calling 'RequestName'")
    resp = await service.RequestName(well_known_bus_name, 4)

    print("[service] reply:", (None, "primary owner", "in queue",
                               "exists", "already owned")[resp[0]])

    await asyncio.sleep(1)

if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['jeepney', 'jeepney.integrate', 'jeepney.tests']

package_data = \
{'': ['*']}

setup(name='jeepney',
      version='0.4',
      description='Low-level, pure Python DBus protocol wrapper.',
      author='Thomas Kluyver',
      author_email='thomas@kluyver.me.uk',
      url='https://gitlab.com/takluyver/jeepney',
      packages=packages,
      package_data=package_data,
      python_requires='>=3.5',
     )



"""Example accessing the SecretStorage DBus interface with asyncio APIs

https://freedesktop.org/wiki/Specifications/secret-storage-spec/secrets-api-0.1.html#ref-dbus-api
"""
import asyncio

from jeepney import new_method_call, DBusAddress, Properties
from jeepney.integrate.asyncio import connect_and_authenticate

secrets = DBusAddress('/org/freedesktop/secrets',
                      bus_name= 'org.freedesktop.secrets',
                      interface='org.freedesktop.Secret.Service')

login_keyring = DBusAddress('/org/freedesktop/secrets/collection/login',
                            bus_name= 'org.freedesktop.secrets',
                            interface='org.freedesktop.Secret.Collection')

msg = new_method_call(login_keyring, 'SearchItems', 'a{ss}', ({
                          'user': 'tk2e15',
                        },)
                     )

async def send_notification():
    (t, p) = await connect_and_authenticate(bus='SESSION')

    resp = await p.send_message(Properties(secrets).get('Collections'))
    print('Collections:', resp[0][1])

    resp = await p.send_message(msg)
    print('Search res:', resp)


loop = asyncio.get_event_loop()
loop.run_until_complete(send_notification())

